package com.xhkjedu.sexam.service.paper;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.sexam.mapper.exam.ESubjectMapper;
import com.xhkjedu.sexam.mapper.paper.*;
import com.xhkjedu.sexam.mapper.system.UserBasketMapper;
import com.xhkjedu.sexam.model.paper.*;
import com.xhkjedu.sexam.vo.paper.*;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.ResultVo;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.awt.*;
import java.util.*;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description 题库题型
 * @Author WN
 * Date 2022/7/19 9:07
 **/
@Service
public class EPaperQtypeService {
    @Resource
    private EPaperQtypeMapper ePaperQtypeMapper;
    @Resource
    private EPaperQtypeQuestionMapper ePaperQtypeQuestionMapper;
    @Resource
    private EPaperMapper ePaperMapper;
    @Resource
    private EPaperAnalyzeMapper ePaperAnalyzeMapper;
    @Resource
    private ESubjectMapper eSubjectMapper;
    @Resource
    private EPaperFileMapper ePaperFileMapper;
    @Resource
    private UserBasketMapper userBasketMapper;


    /**
     * @Description 保存题库试卷
     * @Param [paper]
     * @Return void
     * @Author wn
     * @Date 2022/7/19 9:10
     **/
    @Transactional(rollbackFor = Exception.class)
    public ResultVo saveQtPaper(EPaper paper) {
        //获取考试科目信息
        EExamSubjectPaperVo espVo = ePaperMapper.getPaperParamsByEsid(paper.getEsid());
        if(espVo.getExamstate() > 0){
            return new ResultVo(1,"不能进行操作");
        }else{
            if(espVo.getEpid()!=null){
                //说明已经有改考试对应的试卷，删除原试卷，重新保存
                ePaperMapper.deletePaperByEpid(paper.getEpid());
            }
        }

        List<EPaperQtype> qtypelist = paper.getQtypelist();
        List<EPaperQtypeQuestion> queslist = new ArrayList<>();//存放试题
        setEPaperBaseDetail(paper,espVo,qtypelist,queslist);
        //保存试卷信息
        ePaperMapper.insertUseGeneratedKeys(paper);
        int epid = paper.getId();
        paper.setEpid(epid);
        //保存题型试题信息
        savePaperQtype(paper,qtypelist);

        //保存试卷分析
        if(paper.getPtype() == 1){
            savePaperAnalyzeForQuestion(paper);
        }else{
            List<EPaperFile> pfiles = paper.getPfiles();//附件
            if(paper.getHasfile() == 1){
                setPaperFiles(paper,pfiles);
                ePaperFileMapper.insertList(pfiles);
            }
            savePaperAnalyzeForFj(paper,qtypelist,queslist);
        }

        //更加试卷状态是否需要生成分析和更改科目状态
        if(paper.getPstate()!=null && paper.getPstate() == 1){
            //如果提交试卷，更改科目状态
            eSubjectMapper.updateExamSubjectState(1,espVo.getEsid());
        }
        //保存后清除试题栏中试题
        userBasketMapper.deleteByUseridAndSubjectid(paper.getCreateid(),espVo.getSubjectid());
        return new ResultVo(0,"保存成功",epid);
    }

    //设置试卷基本信息
    private void setEPaperBaseDetail(EPaper paper,EExamSubjectPaperVo eSubject,List<EPaperQtype> qtypelist,List<EPaperQtypeQuestion> queslist){
        qtypelist.forEach(q ->{
            queslist.addAll(q.getQuestions());
        });
        if(paper.getPtype() == 2 && paper.getPfiles()!=null && paper.getPfiles().size()>0){
            paper.setHasfile(1);
        }else {
            paper.setHasfile(0);
        }
        paper.setExamid(eSubject.getExamid());
        paper.setSubjectid(eSubject.getSubjectid());
        paper.setEsid(eSubject.getEsid());
        int timestamp = N_Utils.getSecondTimestamp();
        double pscore = queslist.stream().collect(Collectors.summingDouble(EPaperQtypeQuestion:: getScore));
        paper.setPnum(queslist.size());
        paper.setPscore(pscore);
        paper.setCreatetime(timestamp);
        paper.setHandletime(timestamp);
        paper.setHandleid(paper.getCreateid());
        if(paper.getPtype() == 1){
            paper.setAnswered(1);//题库已设置正确答案
        }else{
            paper.setAnswered(0);
        }
    }

    //保存试卷题型试题信息
    private void savePaperQtype(EPaper epaper,List<EPaperQtype> qtypelist){
        List<EPaperQtypeQuestion> qtqs = new ArrayList<>();
        for(EPaperQtype qtype : qtypelist){
            qtype.setEpid(epaper.getEpid());
            qtype.setCreateid(epaper.getHandleid());
            qtype.setCreatetime(epaper.getHandletime());
            qtype.setEptname(qtype.getEptname().trim());
        }
        ePaperQtypeMapper.insertList(qtypelist);//保存题型
        for(EPaperQtype qtype : qtypelist){
            for(EPaperQtypeQuestion q : qtype.getQuestions()){
                q.setEpid(epaper.getEpid());
                q.setEptid(qtype.getId());
                if(q.getOptionnum()==null){
                    q.setOptionnum(0);
                }
                qtqs.add(q);
            }
        }
        ePaperQtypeQuestionMapper.insertList(qtqs);
    }

    //设置附件
    private void setPaperFiles(EPaper paper,List<EPaperFile> pfiles) {
        if(paper.getHasfile() == 1){
            for(EPaperFile pf :pfiles){
                pf.setCreateid(paper.getCreateid());
                pf.setCreatetime(paper.getHandletime());
                pf.setEpid(paper.getEpid());
            }
        }
    }

    //保存试卷分析--题库
    private void savePaperAnalyzeForQuestion(EPaper ePaper){
        //保存之前先清除之前的试卷分析
        ePaperAnalyzeMapper.deleteByEpid(ePaper.getEpid());
        //总体分析（主观题、客观题）
        List<EPaperQtypeVo> qtypelist = ePaperQtypeMapper.listPaperQtypeQuestions(ePaper.getEpid());//试卷题型试题信息
        List<EPaperQtypeQuestionVo> queslist = new ArrayList<>();//试卷中所有试题(子题)
        List<EPaperQPointVo> quespoints = new ArrayList<>();//试卷中所有单题和子题以及对应知识点

        //处理题型1单选题2多选题3主观题4判断对错5判断√×6判断TF7完形填空8阅读理解10任务型阅读11综合题12听力
        List<EPaperQsVo> typelist = new ArrayList<>();//题型分析
        //处理出试卷中所有试题、知识点
        for(EPaperQtypeVo qt : qtypelist){
            for(EPaperQtypeQuestionVo q : qt.getQuestions()){
                //处理试题关联知识点
                List<EPaperQPointVo> pointslist = q.getPoints();
                if(pointslist != null && pointslist.size()>0){
                    for(EPaperQPointVo point : pointslist){
                        EPaperQPointVo p = new EPaperQPointVo();
                        p.setPointid(point.getPointid());
                        p.setPointname(point.getPointname());
                        p.setQuestionid(q.getQuestionid());
                        p.setQorder(q.getQorder());
                        p.setScore(q.getScore());
                        quespoints.add(p);
                    }
                }
                if(q.getQlevel()==1){
                    q.setQuestionpid("000");
                }
                q.setCtype(q.getQctype());
                queslist.add(q);
            }

            double storerate = N_Utils.getDoubleDivideAndMulitiply(qt.getEptscore(), ePaper.getPscore());
            EPaperQsVo tvo = new EPaperQsVo();
            tvo.setQtname(qt.getEptname());
            tvo.setNum(qt.getEptnum());
            tvo.setScore(qt.getEptscore());
            tvo.setSrate(storerate);
            typelist.add(tvo);
        }

        //获取出所有单题和母题,进行判断区分
        Map<String,List<EPaperQtypeQuestionVo>> quesMap = queslist.stream().collect(Collectors.groupingBy(EPaperQtypeQuestionVo:: getQuestionpid, Collectors.toList()));
        List<EPaperQtypeQuestionVo> mquestions = new ArrayList<>();//仅存放单题和母题
        for(Map.Entry<String, List<EPaperQtypeQuestionVo>> entry : quesMap.entrySet()){
            if(entry.getKey().equals("000")){
                for(EPaperQtypeQuestionVo dq : entry.getValue()){
                    if(N_Utils.isObjectiveQuestion(dq.getCtype())){
                        //判断试题是否是客观题,暂时用mctype进行标识
                        dq.setMctype(111);
                    }else{
                        dq.setMctype(112);
                    }
                    mquestions.add(dq);
                }
            }else{
                EPaperQtypeQuestionVo mq = new EPaperQtypeQuestionVo();
                List<EPaperQtypeQuestionVo> sonques = entry.getValue();
                int k = 0;//如果k大于1说明有小题是主观题，则母题为主观题
                double score = 0;
                for(EPaperQtypeQuestionVo sq : sonques){
                    if(!N_Utils.isObjectiveQuestion(sq.getCtype())){
                        k++;
                    }
                    score = N_Utils.getDoubleSum(score,sq.getScore());
                }

                EPaperQtypeQuestionVo qobj = sonques.get(0);
                mq.setQuestionid(qobj.getQuestionpid());
                mq.setComplexity(qobj.getMcomplexity());
                mq.setScore(score);
                if(k == 0){
                    mq.setMctype(111);
                }else{
                    mq.setMctype(112);
                }
                mquestions.add(mq);
            }
        }

        //试题总体分布进行分析
        //客观题集合
        List<EPaperQtypeQuestionVo> ktlst = mquestions.stream().filter(q->q.getMctype() == 111).collect(Collectors.toList());
        int ktnum = ktlst.size();
        double ktscore = ktlst.stream().collect(Collectors.summingDouble(EPaperQtypeQuestionVo:: getScore));
        double ktnrate = N_Utils.getIntegerDivideAndMulitiply(ktnum, ePaper.getPnum());
        double ktsrate = N_Utils.getDoubleDivideAndMulitiply(ktscore, ePaper.getPscore());
        List<EPaperQsVo> ranglist = new ArrayList<>();
        ranglist.add(new EPaperQsVo("客观题",ktscore,ktnum,ktsrate,ktnrate));
        EPaperQsVo zgq = new EPaperQsVo();
        zgq.setQtname("主观题");
        zgq.setNum(ePaper.getPnum() - ktnum);
        zgq.setNrate(N_Utils.getDoubleReduce(100.0,ktnrate));
        zgq.setScore(N_Utils.getDoubleReduce(ePaper.getPscore(),ktscore));
        zgq.setSrate(N_Utils.getDoubleReduce(100.0,ktsrate));
        ranglist.add(zgq);
        Map ztmap = new HashMap<>();
        ztmap.put("pnum", ePaper.getPnum());
        ztmap.put("pscore",ePaper.getPscore());
        ztmap.put("ranglist",ranglist);

        //难易度
        Map<Integer,List<EPaperQtypeQuestionVo>> complxmap =  mquestions.stream().collect(Collectors.groupingBy(EPaperQtypeQuestionVo:: getComplexity, Collectors.toList()));
        List<Map> clist = new ArrayList<>();
        for(Map.Entry<Integer, List<EPaperQtypeQuestionVo>> entry : complxmap.entrySet()){
            Map cmap = new HashMap();
            cmap.put("level",entry.getKey());
            List<EPaperQtypeQuestionVo> aq = entry.getValue();
            double cscore = aq.stream().collect(Collectors.summingDouble(EPaperQtypeQuestionVo:: getScore));
            double scorerate = N_Utils.getDoubleDivideAndMulitiply(cscore, ePaper.getPscore());
            cmap.put("score",cscore);
            cmap.put("srate",scorerate);
            clist.add(cmap);
        }

        //知识点
        Map<String,List<EPaperQPointVo>> pointmap =  quespoints.stream().collect(Collectors.groupingBy(EPaperQPointVo:: getPointid, Collectors.toList()));
        List<Map> pointlist = new ArrayList<>();
        for(Map.Entry<String, List<EPaperQPointVo>> entry : pointmap.entrySet()){
            Map pm = new HashMap();
            List<EPaperQPointVo> pvo = entry.getValue();
            double cscore = pvo.stream().collect(Collectors.summingDouble(EPaperQPointVo:: getScore));
            double scorerate = N_Utils.getDoubleDivideAndMulitiply(cscore, ePaper.getPscore());
            List<Integer> thlist = pvo.stream().map(EPaperQPointVo :: getQorder).collect(Collectors.toList());
            pm.put("pointname",pvo.get(0).getPointname());
            pm.put("score",cscore);
            pm.put("srate",scorerate);
            pm.put("order",thlist.stream().distinct().collect(Collectors.toList()));
            pointlist.add(pm);
        }

        EPaperAnalyze epa = new EPaperAnalyze();
        epa.setEpid(ePaper.getEpid());
        epa.setAlljson(JSON.toJSONString(ztmap));//总体分析
        epa.setQnumjson(JSON.toJSONString(typelist));//试题栏题型分析
        epa.setComplexityjson(JSON.toJSONString(clist));//难易度分析
        epa.setPointjson(JSON.toJSONString(pointlist));//知识点分析
        ePaperAnalyzeMapper.insertUseGeneratedKeys(epa);

    }

    //保存试卷分析--附件
    private void savePaperAnalyzeForFj(EPaper ePaper,List<EPaperQtype> fjtypelist,List<EPaperQtypeQuestion> queslist){
        //保存之前先清除之前的试卷分析
        ePaperAnalyzeMapper.deleteByEpid(ePaper.getEpid());
        //总体分析（主观题、客观题）
        //处理题型1单选题2多选题3主观题4判断对错5判断√×6判断TF7完形填空8阅读理解10任务型阅读11综合题12听力
        int znum = 0;
        double zscore = 0;
        int knum = 0;
        double kscore = 0;
        for(EPaperQtypeQuestion q : queslist){
            if(N_Utils.isObjectiveQuestion(q.getCtype())){
                knum ++;
                kscore = kscore + q.getScore();
            }else{
                znum ++;
                zscore = zscore + q.getScore();
            }
        }

        double znumrate = N_Utils.getIntegerDivideAndMulitiply(znum, ePaper.getPnum());
        double knumrate = N_Utils.getDoubleReduce(100.0,znumrate);;

        double zscorerate = N_Utils.getDoubleDivideAndMulitiply(zscore, ePaper.getPscore());
        double kscorerate = N_Utils.getDoubleReduce(100.0,zscorerate);

        List<EPaperQsVo> ranglist = new ArrayList<>();
        ranglist.add(new EPaperQsVo("主观题",zscore,znum,zscorerate,znumrate));
        ranglist.add(new EPaperQsVo("客观题",kscore,knum,kscorerate,knumrate));

        Map ztmap = new HashMap<>();
        ztmap.put("pnum", ePaper.getPnum());
        ztmap.put("pscore",ePaper.getPscore());
        ztmap.put("ranglist",ranglist);

        //题型分布(按照名称进行分组)
        fjtypelist.stream().sorted(Comparator.comparing(EPaperQtype::getEptorder)).collect(Collectors.toList());
        Map<String,List<EPaperQtype>> qtypemap =  fjtypelist.stream().collect(Collectors.groupingBy(EPaperQtype:: getEptname, Collectors.toList()));
        List<EPaperQsVo> qtlist = new ArrayList<>();
        for(Map.Entry<String, List<EPaperQtype>> entry : qtypemap.entrySet()){
            List<EPaperQtype> aq = entry.getValue();
            int num = aq.stream().mapToInt(EPaperQtype::getEptnum).sum();
            double tscore = aq.stream().collect(Collectors.summingDouble(EPaperQtype:: getEptscore));
            double storerate = N_Utils.getDoubleDivideAndMulitiply(tscore, ePaper.getPscore());
            EPaperQsVo qvo = new EPaperQsVo();
            qvo.setQtname(entry.getKey());
            qvo.setNum(num);
            qvo.setScore(tscore);
            qvo.setSrate(storerate);
            qtlist.add(qvo);
        }

        EPaperAnalyze epa = new EPaperAnalyze();
        epa.setEpid(ePaper.getEpid());
        epa.setAlljson(JSON.toJSONString(ztmap));
        epa.setQnumjson(JSON.toJSONString(qtlist));
        ePaperAnalyzeMapper.insertUseGeneratedKeys(epa);
    }

    /**
     * @Description 修改试卷
     * @Param [paper]
     * @Return void
     * @Author wn
     * @Date 2022/7/20 8:43
     **/
    @Transactional(rollbackFor = Exception.class)
    public ResultVo updateQtPaper(EPaper paper) {

        //获取考试科目信息
        EExamSubjectPaperVo espVo = ePaperMapper.getPaperParamsByEsid(paper.getEsid());
        if(espVo.getExamstate() >0){
            return new ResultVo(1,"不能进行操作");
        }

        //删除原试卷
        ePaperMapper.deletePaperByEpid(paper.getEpid());

        List<EPaperQtype> qtypelist = paper.getQtypelist();
        List<EPaperQtypeQuestion> queslist = new ArrayList<>();//存放试题

        setEPaperBaseDetail(paper,espVo,qtypelist,queslist);
        //保存试卷信息
        ePaperMapper.insertUseGeneratedKeys(paper);
        int epid = paper.getId();
        paper.setEpid(epid);
        //保存题型试题信息
        savePaperQtype(paper,qtypelist);

        //保存试卷分析
        if(paper.getPtype() == 1){
            savePaperAnalyzeForQuestion(paper);
        }else{
            List<EPaperFile> pfiles = paper.getPfiles();//附件
            if(paper.getHasfile() == 1){
                setPaperFiles(paper,pfiles);
                ePaperFileMapper.insertList(pfiles);
            }
            savePaperAnalyzeForFj(paper,qtypelist,queslist);
        }

        //更加试卷状态是否需要生成分析和更改科目状态
        if(paper.getPstate()!=null && paper.getPstate() == 1){
            //如果提交试卷，更改科目状态
            eSubjectMapper.updateExamSubjectState(1,espVo.getEsid());
        }
        return new ResultVo(0,"保存成功");
    }

    /**
     * @Description  试卷详情
     * @Param [epid]
     * @Return java.util.Map
     * @Author wn
     * @Date 2022/7/19 9:10
     **/
    public Map getPaperQtDetail(Integer esid) {
        Map map = ePaperMapper.getExamPaperDetailByEsid(esid);//试卷信息
        List<EPaperQtypeVo> fjtypelist = new ArrayList<>();
        List<Map> files = new ArrayList<>();
        if(map.get("epid") != null){
            Integer epid = Integer.parseInt(map.get("epid").toString()) ;
            Integer ptype = Integer.parseInt(map.get("ptype").toString()) ;//试卷类型1题库2附件
            if(ptype == 1){
                fjtypelist = ePaperQtypeMapper.listPaperQtypeQuestions(epid);//试卷题型试题信息（题库）
                setEPaperQtypeForQues(fjtypelist);

            }else{
                fjtypelist = ePaperQtypeMapper.listPaperQtypeQuesitonsForFj(epid);//试卷题型试题信息（附件）
                files = ePaperFileMapper.listPaperFiles(epid);

            }
        }
        map.put("qtypes",fjtypelist);
        map.put("pfiles",files);
        return map;
    }

    private void setEPaperQtypeForQues(List<EPaperQtypeVo> fjtypelist){
        for(EPaperQtypeVo qt : fjtypelist){
            List<EPaperQtypeQuestionVo> qlist = qt.getQuestions();
            List<EPaperQtypeQuestionVo> rtnqlist = new ArrayList<>();//保存处理后的试题
            String qpid = "";//母题id
            List<EPaperQtypeQuestionVo> sonqlist = new ArrayList<>();
            for(EPaperQtypeQuestionVo q : qlist){
                //试题类型1单题2母题3子题
                if(q.getQlevel() == 3){
                   if(!q.getQuestionpid().equals(qpid)){
                       sonqlist = qlist.stream().filter(o -> o.getQuestionpid().equals(q.getQuestionpid())).collect(Collectors.toList());
                       EPaperQtypeQuestionVo mq = new EPaperQtypeQuestionVo();
                       mq.setQuestionid(q.getQuestionpid());
                       mq.setCtype(q.getMctype());
                       mq.setQtypeid(q.getMqtypeid());
                       mq.setQtypename(q.getMqtypename());
                       mq.setQstem(q.getMqstem());
                       mq.setComplexity(q.getMcomplexity());
                       List<EPaperQPointVo> allpoint = new ArrayList<>();
                       for(EPaperQtypeQuestionVo sq : sonqlist){
                           List<EPaperQPointVo> sqpoints = sq.getPoints();
                           for(EPaperQPointVo p : sqpoints){
                               if(!isExistEPaperPoints(allpoint,p)){
                                   allpoint.add(p);
                               }
                           }
                           sq.setCtype(sq.getQctype());
                       }
                       mq.setSonques(sonqlist);
                       mq.setPoints(allpoint);
                       rtnqlist.add(mq);
                   }
                    qpid = q.getQuestionpid();
                }else{
                    EPaperQtypeQuestionVo mq = new EPaperQtypeQuestionVo();
                    BeanUtils.copyProperties(q,mq);
                    mq.setCtype(mq.getQctype());
                    rtnqlist.add(mq);
                }
            }
            qt.setQuestions(rtnqlist);
        }

    }

    private boolean isExistEPaperPoints(List<EPaperQPointVo> allpoint,EPaperQPointVo p){
        for(EPaperQPointVo pvo : allpoint){
            if(p.getPointid().equals(pvo.getPointid())){
                return true;
            }
        }
        return false;
    }

    /**
     * @Description 设置试卷正确答案
     * @Param [ePaper]
     * @Return void
     * @Author wn
     * @Date 2022/7/25 15:12
     **/
    @Transactional(rollbackFor = Exception.class)
    public void saveQuesitonAnswer(EPaper ePaper){
        List<EPaperQtypeQuestion> questions = ePaper.getQuestions();
        ePaperQtypeMapper.updateBatchQuestionAnswer(questions);
        ePaper.setAnswered(1);
        ePaperMapper.updateEPaperAnswered(ePaper);
        //仅保存试卷基本信息，后续需要判断发布考试后更改
        //获取考试对应的状态

    }

    /**
     * @Description 获取制作答题卡需要的试卷信息
     * @Param [epid]
     * @Return java.util.Map
     * @Author wn
     * @Date 2022/7/22 10:19
     **/
    public Map getPaperAndQuesitons(Integer epid){
        Map eps = ePaperMapper.getExamPaperAndSubject(epid);//考试考试科目名称
        List<Map> qtypes = new ArrayList<>();//题型试题
        if(eps!=null && eps.get("ptype")!=null){
            int ptype = Integer.parseInt(eps.get("ptype").toString());//试卷类型1题库2附件
            qtypes = ePaperQtypeMapper.listPaperqtypeQuestionsForScan(epid);
            if(ptype == 1){
                //处理试卷中试题选项数量
                for(Map qtm : qtypes){
                    List<Map> qlist = (List<Map>) qtm.get("questions");
                    for(Map qm : qlist){
                        String qoption = qm.get("qoption").toString();
                        Integer ctype = Integer.parseInt(qm.get("qctype").toString());
                        qm.put("ctype",ctype);
                        if(ctype == 1 || ctype ==2 || ctype==4 || ctype ==5 || ctype==6 ){
                            List<String> strlst = JSON.parseArray(qoption,String.class);
                            qm.put("qoption","");
                            qm.put("optionnum",strlst.size());
                        }else if(ctype == 7 || ctype == 8 || ctype ==10){
                            //完形填空、阅读理解、任务型阅读
                            List<QuestionOptionVo> qopt = JSON.parseArray(qoption,QuestionOptionVo.class);
                            qm.put("optionnum",qopt.get(0).getSqoptions().size());
                            qm.put("quenum",qopt.size());
                            qm.put("qoption","");

                        }
                    }
                }
            }
        }

        Map rtnmap = new HashMap();
        rtnmap.put("paper",eps);
        rtnmap.put("qtypes",qtypes);
        return rtnmap;
    }

    /**
     * @Description 保存答题卡
     * @Param [paper]
     * @Return void
     * @Author wn
     * @Date 2022/7/25 17:38
     **/
    @Transactional(rollbackFor = Exception.class)
    public void savePaperScan(EPaper paper){
        List<EPaperQtypeQuestion> questions = paper.getQuestions();
        for(EPaperQtypeQuestion q : questions){
            List<String> coorlist = q.getCoordlist();
            if(q.getCtype() == 3 || q.getCtype() == 14 || q.getCtype() == 15){
                if(coorlist.size() > 1){
                    //有多个坐标，合并为一个
                    List<List> savecoor = new ArrayList<>();
                    for(String str : coorlist){
                        List lst = JSON.parseArray(str);
                        for(int i=0;i<lst.size();i++){
                            savecoor.add((List)lst.get(i));
                        }
                    }
                    q.setCoord(JSON.toJSONString(savecoor));
                }else{
                    q.setCoord(coorlist.get(0));
                }
            }else{
                q.setCoord(coorlist.get(0));
            }
        }
        ePaperMapper.updatePaperScantron(paper);
        ePaperQtypeMapper.updateBatchQuestionForScan(questions);
        if(paper.getPstate() == 1){
            eSubjectMapper.updateEsubjectStateByEpid(1,paper.getEpid());
        }
    }
}
