package com.xhkjedu.sstudy.service.question;

import com.xhkjedu.config.MissingParametersException;
import com.xhkjedu.sstudy.mapper.question.*;
import com.xhkjedu.sstudy.mapper.subjectbook.SubjectBookMapper;
import com.xhkjedu.sstudy.model.question.*;
import com.xhkjedu.sstudy.model.subjectbook.TSubjectBook;
import com.xhkjedu.sstudy.utils.StudyUtil;
import com.xhkjedu.sstudy.vo.subjectbook.DirectorVo;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.sstudy.vo.question.QuestionCollectVo;
import com.xhkjedu.sstudy.vo.question.QuestionVo;
import com.xhkjedu.sstudy.vo.question.SelectQuestionVo;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author ywx
 * @classname QuestionService
 * @description 
 * @date 2020/6/6 17:09
 **/
@Service
public class QuestionService {
    @Resource
    private QuestionMapper questionMapper;
    @Resource
    private QuestionPointMapper questionPointMapper;
    @Resource
    private QuestionCollectMapper questionCollectMapper;
    @Resource
    private QuestionDirectorMapper questionDirectorMapper;
    @Resource
    private SubjectBookMapper subjectBookMapper;
    @Resource
    private UserBasketMapper userBasketMapper;

    //更新单个试题状态
    public Integer updateState(String questionid, Integer qstate) {
        return questionMapper.updateState(questionid, qstate);
    }

    //修改
    public Integer update(TQuestion model) {
        model.setCreatetime(N_Utils.getSecondTimestamp());
        return questionMapper.updateByPrimaryKeySelective(model);
    }

    //获取详情
    public QuestionVo findById(String questionid) {
        return questionMapper.findById(questionid);
    }

    //获取列表
    public List <TQuestion> findAll(TQuestion question) {
        return questionMapper.findAll(question);
    }

    /**
     * 批量更新试题信息
     * @Param [oldQuestion]
     * @Author ywx
     * @Date 2020/6/6 17:01
     * @return java.lang.Integer
     **/
    public Integer updateBatch(List<TQuestion> oldQuestion) {
        return questionMapper.updateBatch(oldQuestion);
    }

    /**
     * 批量保存
     * @Param [addQuestion]
     * @Author ywx
     * @Date 2020/6/6 17:01
     * @return void
     **/
    public void batchSave(List<TQuestion> addQuestion) {
        questionMapper.insertList(addQuestion);
    }

    /**
     * 修改试题状态
     * @Param [qstate, delQuestionId]
     * @Author ywx
     * @Date 2020/6/8 15:48
     * @return void
     **/
    public void updateQuestionState(int qstate, String delQuestionId) {
        questionMapper.updateQuestionState(qstate, delQuestionId);
    }

    /**
     * 试题知识点列表
     * @Param [qs]
     * @Author ywx
     * @Date 2020/6/8 16:37
     * @return java.util.List<com.xhkjedu.vo.question.QuestionVo>
     **/
    public List<QuestionVo> listQuestions(List<QuestionVo> qs) {
        return questionMapper.listQuestions(qs);
    }

    /**
     * 添加
     * @Param [question]
     * @Author ywx
     * @Date 2020/6/8 17:38
     * @return java.lang.Integer
     **/
    public String save(TQuestion question) {
        delQuestionATag(question);
        question.setCount(0);
        question.setQstate(1);
        question.setCreatetime(N_Utils.getSecondTimestamp());
        String questionid = StudyUtil.getId();
        question.setQuestionid(questionid);
        if (N_Utils.isEmpty(question.getSubjectid())) {//科目为空根据教材id取科目
            TSubjectBook book = subjectBookMapper.selectByPrimaryKey(question.getLsbid());
            question.setSubjectid(book.getSubjectid());
        }
        questionMapper.insertSelective(question);

        setQuestionPointDirector(question);
        questionPointMapper.batchSave(question.getQuestionPoints());//保存试题关联知识点信息
        batchSaveQuestionDirector(question.getDirectors());//保存试题关联章节信息
        return questionid;
    }

    /**
     * 修改
     * @Param [question]
     * @Author ywx
     * @Date 2020/6/9 8:33
     * @return void
     **/
    public void updateQuestion(TQuestion question) {
        N_Utils.validation(new Object[]{
                question.getQstem(),"试题题干",2,question.getQanswer(),"试题答案",2
                ,question.getScore(),"试题分值",1,question.getComplexity(),"试题难易度",1
                ,question.getQtypeid(),"试题题型id",2,question.getQuestionid(),"试题id",2
        });
        if (CollectionUtils.isEmpty(question.getQuestionPoints())){
            throw new MissingParametersException("试题关联知识点不能为空");
        }
        // 修改 试题_知识点关联信息
        questionPointMapper.deleteByQuestionId(question.getQuestionid());//删除试题关联知识点信息
        questionDirectorMapper.deleteByQuestionId(question.getQuestionid());//删除试题关联章节信息
        setQuestionPointDirector(question);
        questionPointMapper.batchSave(question.getQuestionPoints());//保存试题关联知识点信息
        batchSaveQuestionDirector(question.getDirectors());//保存试题关联章节信息
        delQuestionATag(question);
        update(question);
    }

    /**
     * 设置试题知识点、章节关联信息
     * @Param [question]
     * @Author ywx
     * @Date 2020/6/9 8:38
     * @return void
     **/
    public void setQuestionPointDirector(TQuestion question) {
        String questionid = question.getQuestionid();
        Integer createid = question.getCreateid();
        Integer schoolid = question.getSchoolid();
        List<TQuestionPoint> questionPoints = question.getQuestionPoints();
        for (TQuestionPoint qp : questionPoints) {
            qp.setQuestionid(questionid);
            qp.setCreateid(createid);
            qp.setCreatetime(N_Utils.getSecondTimestamp());
            qp.setSchoolid(schoolid);
            qp.setQpid(StudyUtil.getId());
        }
        question.setQuestionPoints(questionPoints);

        //章节
        List<TQuestionDirector> directors = question.getDirectors();
        if (N_Utils.isListEmpty(directors)) {//兼容以前添加单个章节模式
            directors = new ArrayList<>();
            TQuestionDirector qd = new TQuestionDirector();
            qd.setLsbid(question.getLsbid());
            qd.setDirectorid(question.getDirectorid());
            directors.add(qd);
        }
        for (TQuestionDirector qd : directors) {
            qd.setQuestionid(questionid);
            qd.setCreateid(createid);
            qd.setCreatetime(N_Utils.getSecondTimestamp());
            qd.setQdid(StudyUtil.getId());
        }
        question.setDirectors(directors);
    }

    /**
     * 批量保存试题章节关联信息
     * @Param [directors]
     * @Author ywx
     * @Date 2020/12/30 13:43
     * @return void
     **/
    public void batchSaveQuestionDirector(List<TQuestionDirector> directors){
        if (N_Utils.isListNotEmpty(directors)) {
            questionDirectorMapper.batchSave(directors);
        }
    }

    /**
     * @author ywx
     * @description 去除试题中的a标签
     * @date 2020/8/26 20:34
     **/
    private void delQuestionATag(TQuestion question) {
        question.setQstem(N_Utils.delATag(question.getQstem()));
        question.setQoption(N_Utils.delATag(question.getQoption()));
        question.setQanswer(N_Utils.delATag(question.getQanswer()));
        question.setQanalyze(N_Utils.delATag(question.getQanalyze()));
        if(question.getCtype() == 11 || question.getCtype() == 12){
            //说明改题有子题
            List<TQuestion>  sonques = question.getQuestions();//子题
            for(TQuestion sq : sonques){
                sq.setQstem(N_Utils.delATag(sq.getQstem()));
                sq.setQoption(N_Utils.delATag(sq.getQoption()));
                sq.setQanswer(N_Utils.delATag(sq.getQanswer()));
                sq.setQanalyze(N_Utils.delATag(sq.getQanalyze()));
            }
        }
    }

    /**
     * 设置试题知识点关联信息
     * @Param [question]
     * @Author ywx
     * @Date 2020/6/9 8:38
     * @return void
     **/
    public List<TQuestionPoint> setQuestionPoint(TQuestion question) {
        String questionid = question.getQuestionid();
        Integer createid = question.getCreateid();
        Integer schoolid = question.getSchoolid();
        List<TQuestionPoint> questionPoints = question.getQuestionPoints();
        for (TQuestionPoint qp : questionPoints) {
            qp.setQuestionid(questionid);
            qp.setCreateid(createid);
            qp.setCreatetime(N_Utils.getSecondTimestamp());
            qp.setSchoolid(schoolid);
        }
        return questionPoints;
    }


    /**
     * 章节、知识点选题
     * @Param [question]
     * @Author ywx
     * @Date 2020/6/9 15:00
     * @return java.util.List<com.xhkjedu.vo.question.SelectQuestionVo>
     **/
    public List<SelectQuestionVo> listForSelect(TQuestion question) {
        return questionMapper.listForSelect(question);
    }

    /**
     * 试题收藏
     * @Param [collect]
     * @Author ywx
     * @Date 2020/8/18 16:00
     * @return java.lang.Integer
     **/
    public Integer saveCollect(TQuestionCollect collect) {
        collect.setCreatetime(N_Utils.getSecondTimestamp());
        questionCollectMapper.insertUseGeneratedKeys(collect);
        return collect.getId();
    }

    /**
     * 试题取消收藏
     * @Param [collect]
     * @Author ywx
     * @Date 2020/8/18 16:01
     * @return void
     **/
    public void cancelCollect(TQuestionCollect collect) {
        questionCollectMapper.cancelCollect(collect);
    }

    /**
     * 获取试题是否收藏列表
     * @Param [questionids]
     * @Author ywx
     * @Date 2020/8/18 16:14
     * @return java.util.List<com.xhkjedu.vo.question.QuestionCollectVo>
     **/
    public List<QuestionCollectVo> listCollect(List<String> questionids, Integer userid) {
        List<String> questionIds = questionCollectMapper.listCollect(questionids, userid);
        List<QuestionCollectVo> list = new ArrayList<>();
        for (String questionid : questionids) {
            QuestionCollectVo qc = new QuestionCollectVo();
            qc.setQuestionid(questionid);
            Integer collected = 0;
            if (N_Utils.strInList(questionid, questionIds)){
                collected = 1;
            }
            qc.setCollected(collected);
            list.add(qc);
        }
        return list;
    }

    /**
     * @Description 保存编辑试题，编辑后保存为新试题默认为个人题库
     * @Param [question]
     * @Return java.lang.String
     * @Author wn
     * @Date 2022/7/19 16:02
     **/
    @Transactional(rollbackFor = Exception.class)
    public String saveUpdateQuestion(TQuestion question){
        QuestionVo oldQuestion = questionMapper.findById(question.getQuestionid());//原试题详细信息
        delQuestionATag(question);//优化试题内容
        question.setSubjectid(oldQuestion.getSubjectid());

        List<TQuestion> saveQues = new ArrayList<>();//试题集合
        List<TQuestionPoint> saveQPoints = new ArrayList<>();//试题知识集合
        List<TQuestionDirector> saveQDirs = new ArrayList<>();//试题章节集合

        setSaveUpdateQuestion(question,saveQues,saveQPoints);
        setSaveUpdateQuesitonDirector(oldQuestion,question,saveQDirs);
        questionMapper.saveBathQuestion(saveQues);//保存试题
        questionPointMapper.batchSave(saveQPoints);//保存试题关联知识点信息
        questionDirectorMapper.batchSave(saveQDirs);//保存试题关联章节

        //删除试题栏中原题，加入新题
       /* userBasketMapper.deleteByQuestioniduid(question.getCreateid(),oldQuestion.getQuestionid());
        TUserBasket ub = new TUserBasket();
        ub.setUserid(question.getCreateid());
        ub.setSubjectid(question.getSubjectid());
        ub.setQtypeid(question.getQtypeid());
        ub.setQtypename(question.getQtypename());
        ub.setQuestionid(question.getQuestionid());
        ub.setCreatetime(question.getCreatetime());
        userBasketMapper.insertSelective(ub);//保存新试题进入试题栏*/

        return question.getQuestionid();
    }

    //处理试题试题和知识
    private void setSaveUpdateQuestion(TQuestion question,List<TQuestion> saveQues,List<TQuestionPoint> saveQPoints){
        int timestamp = N_Utils.getSecondTimestamp();
        question.setCount(0);
        question.setQstate(1);
        question.setBelong(4);
        question.setCreatetime(timestamp);
        String questionid = StudyUtil.getId();
        question.setQuestionid(questionid);
        saveQues.add(question);
        //qlevel试题类型1单体2母题3子题
        if(question.getQlevel() == 2){//是母体并且包含子题
            List<TQuestion> sonques = question.getQuestions();
            for(TQuestion sq : sonques){
                String sqid = StudyUtil.getId();
                sq.setQuestionid(sqid);
                sq.setCount(0);
                sq.setQstate(1);
                sq.setBelong(4);
                sq.setQuestionpid(questionid);
                sq.setSubjectid(question.getSubjectid());
                sq.setCreateid(question.getCreateid());
                sq.setCreatetime(timestamp);
                sq.setSchoolid(question.getSchoolid());
                saveQues.add(sq);

                //知识点
                List<TQuestionPoint> points = sq.getQuestionPoints();
                if(points!=null && points.size()>0){
                    for(TQuestionPoint p : points){
                        TQuestionPoint mp = new TQuestionPoint();
                        String mqpid = StudyUtil.getId();
                        mp.setQpid(mqpid);
                        mp.setQuestionid(questionid);
                        mp.setPointid(p.getPointid());
                        mp.setCreateid(question.getCreateid());
                        mp.setCreatetime(timestamp);
                        saveQPoints.add(mp);//母题关联所有子题关联知识点

                        //子题关联知识点
                        String qpid = StudyUtil.getId();
                        p.setQpid(qpid);
                        p.setQuestionid(sqid);
                        p.setCreateid(question.getCreateid());
                        p.setCreatetime(timestamp);
                        p.setSchoolid(question.getSchoolid());
                        saveQPoints.add(p);//子题关联知识点
                    }
                }
            }
        }else{
            //单题型母题关联知识点
            //知识点
            List<TQuestionPoint> points = question.getQuestionPoints();
            if(points!=null && points.size()>0){
                for(TQuestionPoint p : points){
                    p.setQpid(StudyUtil.getId());
                    p.setQuestionid(questionid);
                    p.setCreateid(question.getCreateid());
                    p.setCreatetime(timestamp);
                    p.setSchoolid(question.getSchoolid());
                    saveQPoints.add(p);
                }
            }
        }
    }


    //设置章节
    private void setSaveUpdateQuesitonDirector(QuestionVo qvo,TQuestion question,List<TQuestionDirector> saveQDirs){
        //原试题关联章节
        List<DirectorVo> dirs = qvo.getDirectors();
        if(dirs!=null && dirs.size()>0){
            for(DirectorVo d : dirs){
                TQuestionDirector qd = new TQuestionDirector();
                qd.setQdid(StudyUtil.getId());
                qd.setQuestionid(question.getQuestionid());
                qd.setLsbid(d.getLsbid());
                qd.setDirectorid(d.getDirectorid());
                qd.setCreateid(question.getCreateid());
                qd.setCreatetime(question.getCreatetime());

                saveQDirs.add(qd);
            }
        }
    }
}
